#-----------------------------------------------------------------------------
# Vehicle steering script, version 3, with banking and "diving"
# by D. Drumright (Ockham)
# 09/09/03
#
# This script is meant to be applied to an entire animation sequence
# AFTER you have finished pushing the Vehicle around manually, using
# the Xtrans, Ytrans and Ztrans dials or the Translation tool to set its
# position at various frames.  (Assuming normal spline behavior, so
# that Poser will curve the translational movement between your
# manually specified steps.)
#
# After applying the script, the Vehicle's Y rotation will then turn
# appropriately at each frame so that the Vehicle appears to be
# steering into each turn.  The Vehicle will also lean (bank)
# to the degree you specify in the function call.
#
# Note 1: This works on Currently Selected Actor.
#
# Note 2: Because of the "gimbal lock" problem in Poser, this
# works best if you use the Carrier trick, as follows:
#
# 2A. Set up a simple Box prop in default orientation.  Move it,
# without any rotation, to the start position for your figure.
#
# 2B. This Box is the carrier, and will actually take the movements
# as set by this script.
#
# 2C. Move your figure to that same start position, and rotate it
# to the desired starting angle.
#
# 2D. Parent the figure to the Box.
#
# 2E. Set your animation frames, and then move the Box around
# the scene without any rotation; the Figure will be carried
# by the Box.
#
# 2F. Select the Box (not the Figure!) before applying this script.
#
#
# Note 3: See the function calls in the last few lines of
# this file.  The number for Steer determines the strength of "roll",
# and the number in the Dive function determines the strength
# of "pitch".
#
#-----------------------------------------------------------------------------

import poser
import math
global OldY
OldY = 0.0  # Holder for deltas

scene = poser.Scene()

# Find the Platform and Vehicle.
Platform = scene.CurrentActor()
VL = Platform.Children()
try:
    Vehicle = VL[0]
except:
    raise "Vehicle is not parented to Platform."

#-----------------------------------------------------------------------------


def Steer(thisframe,BankConstant):
    global OldY
    # Apply the Yrot to the Platform, and the other two rots to the Vehicle.
    scene.SetFrame(thisframe) # Put everything into first position
    PresentOrigin = Platform.WorldDisplacement() # Where is the object in first position?
    aheadframe = thisframe + 5 # second position is further ahead
    if aheadframe < scene.NumFrames() : # protect against end of list
       scene.SetFrame(aheadframe) # Put everything into the new position
       AheadOrigin = Platform.WorldDisplacement() # Where is the Vehicle in new position?
       Xdist = AheadOrigin[0] - PresentOrigin[0]  # Figure X distance old to new
       Zdist = AheadOrigin[2] - PresentOrigin[2]  # Figure Z distance old to new
       RawY = math.atan2(-Zdist,Xdist)  # Angle of vector from old to new

       scene.SetFrame(thisframe)  # Come back to where we really are
       #Now set the Y rotation of the Platform to anticipate this direction

       NewY = RawY * 180.0 / math.pi # Turn to degrees

       alterParm = Platform.ParameterByCode(poser.kParmCodeYROT)
       alterParm.SetValue(NewY+90.0) # Set Y rotation in degrees

       # Now set the banking (roll) of the Vehicle itself.
       DeltaY = NewY - OldY
       Bank = BankConstant * math.sin(DeltaY)
       zParm = Vehicle.ParameterByCode(poser.kParmCodeZROT)
       zParm.SetValue(Bank)

       OldY = NewY  # Hold for next


#-----------------------------------------------------------------------------

def Dive(thisframe,BankConstant):

    scene.SetFrame(thisframe) # Put everything into first position --
    PresentOrigin = Platform.WorldDisplacement() # Where is the Platform in first position?
    aheadframe = thisframe + 5 # second position is further ahead
    if aheadframe < scene.NumFrames() : # protect against end of list
       scene.SetFrame(aheadframe) # Put everything into the new position
       AheadOrigin = Platform.WorldDisplacement() # Where is the Platform in new position?
       scene.SetFrame(thisframe)  # Come back to where we really are

       # Figure diffs for each:
       Xdist = float(AheadOrigin[0] - PresentOrigin[0])
       Ydist = float(AheadOrigin[1] - PresentOrigin[1])
       Zdist = float(AheadOrigin[2] - PresentOrigin[2])

       # Net horizontal distance:
       HorDist = math.hypot(Xdist,Zdist)

       # Save from divide by zero....
       if HorDist == 0.0:
           DeltaPitch = 0.0
       else:
           # Slope is arctangent of Ydist / Hordist.
           DeltaPitch = math.atan2(-Ydist,HorDist)

       FinalDelta = DeltaPitch * BankConstant * 180.0 / math.pi
       xParm = Vehicle.ParameterByCode(poser.kParmCodeXROT)
       xParm.SetValue(FinalDelta)

#-----------------------------------------------------------------------------
# Main loop to process each frame
#-----------------------------------------------------------------------------

scene = poser.Scene()

# First apply the Banking, all the way through the frames.
for i in range(scene.NumFrames()):
  Steer(i, -15.0) # Set the number to control strength of Banking

# Now start from the beginning again and apply the Dive actions.
for i in range(scene.NumFrames()):
  Dive(i, 0.4) # Set the number to control strength of Diving

#-----------------------------------------------------------------------------
